;------------------------------------------------------------------------------
;  definitions.asm
;  - shared definitions across modules
;
;  Andrew Cooper
;  http://www.darkerview.com
;  CC BY-SA 4.0 - Attribution-ShareAlike 4.0 International
;
;  12Jan2017 A. Cooper
;  - initial version

;------------------------------------------------------------------------------
; hardware assignments

;user port items
userPort	equ	LATB	;user IO pins (buttons and LED's)
userRead	equ	PORTB
userDir	equ	TRISB
	
;bits in user port
userUIO0	equ	0x0
userUIO1	equ	0x1
userUIO2	equ	0x2
userUIO3	equ	0x3
userLED1	equ	0x4
userLED2	equ	0x5
	
;data bus items
dataPort	equ	LATD	;shared 8bit data bus
dataRead	equ	PORTD
dataDir	equ	TRISD
dispCtlPort	equ	LATF	;port for display control lines
dispCtlDir	equ	TRISF
sel02Port	equ	LATC	;select lines for data read/write
sel02Dir	equ	TRISC
sel35Port	equ	LATE
sel35Dir	equ	TRISE
	
;select bits for SVC port data bus
sel0	equ	0x6
sel1	equ	0x7
sel2	equ	0x2
sel3	equ	0x0
sel4	equ	0x1
sel5	equ	0x2
	
;SSP port items
sspDir	equ	TRISC
SDO	equ	0x5
SDI	equ	0x4
SCK	equ	0x3

;bits in display control port
dispRS	equ	0x0
dispRW	equ	0x1
dispE	equ	0x2
dispRed	equ	0x5
dispGrn	equ	0x6
dispBlu	equ	0x7

;serial port items
serPort	equ	PORTG	;port for EUSART1
serDir	equ	TRISG
serCTS	equ	0x1	;pin for CTS
serRTS	equ	0x2	;pin for RTS
ser2Port	equ	PORTE	;port for EUSART2
ser2Dir	equ	TRISE
serCTS2	equ	0x4	;pin for CTS
serRTS2	equ	0x5	;pin for RTS

;------------------------------------------------------------------------------
; flags in shared registers
	
;bits in userFlag
userPB1	equ	0x0	;user pushbutton 1 pressed flag
userPB2	equ	0x1	;user pushbutton 2 pressed flag
userEncUp	equ	0x2	;encoder count up flag
userEncDn	equ	0x3	;encoder count down flag
userLED1On	equ	0x4	;indicator LED 1 on
userLED1Fl	equ	0x5	;indicator LED 1 flash
userLED2On	equ	0x6	;indicator LED 2 on
userLED2Fl	equ	0x7	;indicator LED 2 flash
	
;bits within serFlag
serOpen	equ	0	;port is active and ready
serCmd	equ	1	;command received, <cr> found
serBufOv	equ	2	;buffer overflow
serBusy	equ	3	;port is busy, <CTS> not active
ser2Open	equ	4	;port2  is active and ready
ser2Cmd	equ	5	;command received, <cr> found
ser2BufOv	equ	6	;buffer 2 overflow
ser2Busy	equ	7	;port 2 is busy, <CTS> not active

;bits in stateFlag
stateUp	equ	0x7	;update the display (1Hz)

;-- end defs.asm --------------------------------------------------------------