;------------------------------------------------------------------------------
;  host.asm
;  - Output routines for com with a serial host
;
;  Andrew Cooper
;  http://www.darkerview.com
;  CC BY-SA 4.0 - Attribution-ShareAlike 4.0 International
;
;  26Apr2010 A.Cooper
;  - Modified for LEDCtrl
;  29Jan2013 A.Cooper
;  - Modified for iSource
;  - Modified for use with PIC16F873 (must banksel all RAM access)
;  - Converted HostChar and HostDChar to macros 
;  30May2015 A. Cooper
;  - rewritten for PIC18F
;  - modified for Incline
;  - changed to relocatable code
;  6Feb2017 A. Cooper
;  - modified for GenPIC
;
;------------------------------------------------------------------------------

;------------------------------------------------------------------------------
;  Global Definitions
	include	PIC18F66K80.inc ;processor specific definitions
	include	"defs.asm"
	
;------------------------------------------------------------------------------
;  Declarations
	UDATA_ACS
msgNum	res	1
msgCnt	res	1
count	res	1
;resLo	res	1
;resHi	res	1
temp	res	1

;------------------------------------------------------------------------------
;  Available Resources
	global	HostMsg, HostHex, HostDec2, HostLDec, HostHexSp
	global	HostMsgRev, HostMsgRst, HostPrompt, Host3
	global	Host4, Host5, Host6, Host7
	global	Host8, Host9, HostA, HostB
	global	HostLine, HostMsgCErr, HostMsgPErr, HostMsgHelp
;	global	resHi, resLo
	
;------------------------------------------------------------------------------
;  External Resources
	extern	SerSend, SerSendDec
	extern	HexToBCD
	extern	resHi, resLo
	
;------------------------------------------------------------------------------
;  Module Code
host	code

;------------------------------------------------------------------------------
;  HostHex
;
;  - send a hexidecimal byte to host
;  - data in W
;
HostHex	movwf	temp,A	;temp save char
	swapf	temp,W,A	;get upper nibble
	andlw	0x0F
	addlw	0xF6	;check for A-F
	btfsc	STATUS,C,A
	 addlw	0x07
	addlw	0x3A	
	call	SerSend	;send it
	movf	temp,W,A	;get lower nibble
	andlw	0x0F
	addlw	0xF6	;check for A-F
	btfsc	STATUS,C,A
	 addlw	0x07
	addlw	0x3A	
	call	SerSend	;send it
	return
HostHexSp	call	HostHex	;HostHex with a following space
	movlw	0x20
	call	SerSend
	return

;------------------------------------------------------------------------------
;  HostDec2
;
;  - send a byte to host in decimal
;  - send only the lower two digits
;  - data in W
;
HostDec2	call	HexToBCD
	swapf	resLo,W,A	;send second digit
	andlw	0x0F
	call	SerSendDec
	movf	resLo,W,A	;send third digit
	andlw	0x0F
	call	SerSendDec
	return
		
;------------------------------------------------------------------------------
;  HostLDec
;  - send two bytes to host in decimal
;  - data in resHi and resLo on entry
;  - leading zeros suppressed, print space instead
;  - uses all four temp registers
;  - Rountine adapted from PIClist by Rich Leggitt with tweaks by Scott Dattalo
;  and bugfix by Dmitry Kiryashov and Nikolai Golovchenko
;  - see: http://www.piclist.com/techref//microchip/math/radix/b2a-16b5a-rl.htm
;
;  - Modified to place a decimal before the last digit!
;    **************************************************
;  - remove the indicated code to delete
;
HostLDec	clrf	count,A	;clear registers
	clrf	temp,A
	goto	HLD1kStart
HLDsub10k	incf	count,F,A
HLD1kStart	movlw	0x10	;=10000 & 255
	subwf	resLo,F,A
	movlw	0x27	;=10000 >> 8
	btfss	STATUS,C,A
	 movlw	0x28	;=(10000>>8)+1
	subwf	resHi,F,A
	bc	HLDsub10k
	movf	count,W,A	;output char
	call	HLDSend
	movlw	0x0A	;=10
	movwf	count,A
HLDadd1k	decf	count,F,A
	movlw	0xE8	;=1000 & 255
	addwf	resLo,F,A
	movlw	0x03	;=1000 >> 8
	btfsc	STATUS,C
	 movlw	0x04	;=(1000>>8)+1
	addwf	resHi,F,A
	bnc	HLDadd1k
        	movf	count,W,A	;send char
	call	HLDSend
	clrf	count,A
	movlw	0x64	;=100
	goto	HLD100Start
HLDsub100	incf	count,F,A
HLD100Start	subwf	resLo,F,A
	btfsc	STATUS,C,A
	 goto	HLDsub100
	decf	resHi,F,A
	btfss	resHi,7	;Check msb instead of carry for underflow.
	 goto	HLDsub100
			;at this point, resHi = 0xff, and  0 <= resLo <= 99
	movf	count,W,A	;output char
	call	HLDSend
	movlw	0x0A	;=10
	movwf	count,A
HLDadd10	decf	count,F,A
	addwf	resLo,F,A
	bnc	HLDadd10
	movf	count,W,A	;send last chars
	call	SerSendDec	;*** insure one char before decimal
	;call	HLDSend
	movlw	'.'	;*** send a decimal before the last digit
	call	SerSend	;*** remove for no decimal
	movf	resLo,W,A
	call	SerSendDec
	return

HLDSend	addlw	0x00	;is it zero?
	btfsc	STATUS,Z	;do leading space suppression
	 goto 	HLDSuppress
	bsf	temp,0	;set stop supressing flag upon first non-zero
	call	SerSendDec
	return
HLDSuppress	btfss	temp,0	;not supressing? send zero
	 movlw	0xF0	;else send space
	call	SerSendDec
	return

;------------------------------------------------------------------------------
;  HostMsg Predefines
;  - send the requested message
;
HostMsgRev	movlw	0x00	
	goto	HostMsg
HostMsgRst	movlw	0x01	
	goto	HostMsg
HostPrompt	movlw	0x02	
	goto	HostMsg
Host3	movlw	0x03	
	goto	HostMsg
Host4	movlw	0x04	
	goto	HostMsg
Host5	movlw	0x05	
	goto	HostMsg
Host6	movlw	0x06	
	goto	HostMsg
Host7	movlw	0x07	
	goto	HostMsg
Host8	movlw	0x08	
	goto	HostMsg
Host9	movlw	0x09	
	goto	HostMsg
HostA	movlw	0x0A	
	goto	HostMsg
HostB	movlw	0x0B	
	goto	HostMsg
HostLine	movlw	0x0C	
	goto	HostMsg
HostMsgCErr	movlw	0x0D	
	goto	HostMsg
HostMsgPErr	movlw	0x0E	
	goto	HostMsg
HostMsgHelp	movlw	0x0F	
	goto	HostMsg

;------------------------------------------------------------------------------
;  HostMessage
;  - send text message to the host
;  - message number in W
;  - currently limited to 16 messages, expand by adding table entries and
;  changing the bit mask on entry
;  - a negative number is used as the end of string flag, thus no extended
;  ASCII chars may be used
;
HostMsg	andlw	0x0F	;qualify input, message #0-15
	movwf	msgNum,A	;store message number
	movlw	UPPER HMText	;set table pointer
	movwf	TBLPTRU,A
	movlw	HIGH HMText
	movwf	TBLPTRH,A
	movlw	LOW HMText
	movwf	TBLPTRL,A
	
	movf	msgNum,W,A	;is message number zero?
	btfsc	STATUS,Z
	 goto	HMOut

HMNext	tblrd*+		;get char from message table
	movf	TABLAT,W,A	
	btfss	STATUS,N,A	;is it end of string (negative)?
	 goto	HMNext
	decfsz	msgNum,F,A	;count strings
	 goto	HMNext
	
HMOut	tblrd*+		;retrieve value from table
	movf	TABLAT,W,A
	btfsc	STATUS,N,A	;is it the end of the string?
	 return
	btfsc	STATUS,Z,A	;skip nulls used to pad data with da
	 goto	HMOut
	call	SerSend	;send it out the serial port
	goto	HMOut

HMRevision	equ	0x00
HMText	da	0x0D0A,0x0A0A
	da	"GenPIC ver 0.2",0x0D0A
	da	"Andrew Cooper",0x0D0A
	da	"http://www.darkerview.com",0x0D0A
	da	"CC BY-SA 4.0 - Attribution-ShareAlike 4.0 International",0x0D0A,0xFF
HMReset	equ	0x01
	da	"GenPIC reset... Ready!",0x0D0A,0xFF
HMPrompt	equ	0x02
	da	"GenPIC>",0xFF
HMsg3	equ	0x03
	da	"",0xFF
HMsg4	equ	0x04
	da	"",0x0D0A,0xFF
HMsg5	equ	0x05
	da	"",0xFF
HMsg6	equ	0x06
	da	"",0xFF
HMsg7	equ	0x07
	da	"",0xFF
HMsg8	equ	0x08
	da	"",0x0D0A,0xFF
HMsg9	equ	0x09
	da	"",0x0D0A,0xFF
HMsgA	equ	0x0A
	da	"",0x0D0A,0xFF
HMsgB	equ	0x0B
	da	"",0x0D0A,0xFF
HMsgLine	equ	0x0C
	da	0x0D0A,0xFF
HMCmdErr	equ	0x0D
	da	"Command Error!!",0x0D0A,0xFF
HMPmErr	equ	0x0E
	da	"Illegal Parameter, out of range!!",0x0D0A,0xFF
HMHelp	equ	0x0F
	da	"GenPIC controller help...",0x0D0A
	da	"  RST     Reset controller",0x0D0A
	da	"  CLK     Report time",0x0D0A
	da	"  ENC XXX Set encoder value to xxx",0x0D0A
	da	"  TMP     Report temperature",0x0D0A
	da	"  HLP     This help page",0x0D0A,0x0AFF

	end
;- end host.asm ---------------------------------------------------------------
