;------------------------------------------------------------------------------
;  message.asm
;  - Output messages for an lcd display
;
;  Andrew Cooper
;  http://www.darkerview.com
;  CC BY-SA 4.0 - Attribution-ShareAlike 4.0 International
;
;  13Jan2017 A. Cooper
;  - modified for use with GenPIC
;
;------------------------------------------------------------------------------

;------------------------------------------------------------------------------
;  Global Definitions
	include	PIC18F66K80.inc ;processor specific definitions
	include	"defs.asm"

;------------------------------------------------------------------------------
;  Declarations
	UDATA_ACS
msgNum	res	1

;------------------------------------------------------------------------------
;  Available Resources
	global	MsgMsg
	global	MsgTest,Msg11,Msg12,Msg13
	global	Msg14,Msg15,Msg16,Msg17
	global	Msg18,Msg18,Msg1A,Msg1B
	global	Msg1C,Msg1D,Msg1E,Msg1F
	
;------------------------------------------------------------------------------
;  External Resources
	extern	DispWrite
	
;------------------------------------------------------------------------------
;  Module Code
message	code
		
;------------------------------------------------------------------------------
;  MsgMsg Predefines
;  - send the requested message
;  - labels can be changed to reflect the message contents
;
MsgTest	movlw	0x10	
	goto	MsgMsg
Msg11	movlw	0x11	
	goto	MsgMsg
Msg12	movlw	0x12	
	goto	MsgMsg
Msg13	movlw	0x13	
	goto	MsgMsg
Msg14	movlw	0x14	
	goto	MsgMsg
Msg15	movlw	0x15	
	goto	MsgMsg
Msg16	movlw	0x16	
	goto	MsgMsg
Msg17	movlw	0x17	
	goto	MsgMsg
Msg18	movlw	0x18	
	goto	MsgMsg
Msg19	movlw	0x19	
	goto	MsgMsg
Msg1A	movlw	0x1A	
	goto	MsgMsg
Msg1B	movlw	0x1B	
	goto	MsgMsg
Msg1C	movlw	0x1C	
	goto	MsgMsg
Msg1D	movlw	0x1D	
	goto	MsgMsg
Msg1E	movlw	0x1E	
	goto	MsgMsg
Msg1F	movlw	0x1F	
	goto	MsgMsg

;------------------------------------------------------------------------------
;  MsgMessage
;  - send text message to the LCD display
;  - message number in W
;  - currently limited to 32 messages, expand by adding table entries and
;  changing the bit mask on entry
;  - a negative number is used as the end of string flag, thus no extended
;  ASCII chars may be used
;  - the first 16 messages are the state titles as used in state.asm
;
MsgMsg	andlw	0x1F	;qualify input, message #0-31
	movwf	msgNum,A	;store message number
	movlw	UPPER MMText	;set table pointer
	movwf	TBLPTRU,A
	movlw	HIGH MMText
	movwf	TBLPTRH,A
	movlw	LOW MMText
	movwf	TBLPTRL,A
	
	movf	msgNum,W,A	;is message number zero?
	btfsc	STATUS,Z
	 goto	MMOut

MMNext	tblrd*+		;get char from message table
	movf	TABLAT,W,A	
	btfss	STATUS,N,A	;is it end of string (negative)?
	 goto	MMNext
	decfsz	msgNum,F,A	;count strings
	 goto	MMNext
	
MMOut	tblrd*+		;retrieve value from table
	movf	TABLAT,W,A
	btfsc	STATUS,N,A	;is it the end of the string?
	 return
	btfsc	STATUS,Z,A	;skip nulls used to pad data with da
	 goto	MMOut
	call	DispWrite	;send it to the LCD display
	goto	MMOut

MMText	da	"GenPIC v0.2",0xFF	;0
	da	"1:Clock",0xFF	;1
	da	"2:Encoder",0xFF	;2
	da	"3:Temp",0xFF		;3
	da	"4:LEDs",0xFF		;4
	da	"State 5",0xFF	;5
	da	"State 6",0xFF	;6
	da	"State 7",0xFF	;7
	da	"State 8",0xFF	;8
	da	"State 9",0xFF	;9
	da	"State A",0xFF	;A
	da	"State B",0xFF	;B
	da	"State C",0xFF	;C
	da	"State D",0xFF	;D
	da	"State E",0xFF	;E
	da	"State F",0xFF	;F
	da	"Test Program",0xFF	;10
	da	"",0xFF		;11
	da	"",0xFF		;12
	da	"",0xFF		;13
	da	"",0xFF		;14
	da	"",0xFF		;15
	da	"",0xFF		;16
	da	"",0xFF		;17
	da	"",0xFF		;18
	da	"",0xFF		;19
	da	"",0xFF		;1A
	da	"",0xFF		;1B
	da	"",0xFF		;1C
	da	"",0xFF		;1D
	da	"",0xFF		;1E
	da	"",0xFF		;1F

	end
;- end message.asm ------------------------------------------------------------
