;------------------------------------------------------------------------------
;  Misc
;
;  - various utility routines (delays, conversions, and math)
;
;  Andrew Cooper
;  http://www.darkerview.com
;  CC BY-SA 4.0 - Attribution-ShareAlike 4.0 International
;
;  23May2015 A. Cooper
;  - initial version
;
;------------------------------------------------------------------------------

;------------------------------------------------------------------------------
;  Global Definitions
	include	PIC18F66K80.inc ;processor specific definitions
	include	"defs.asm"

;------------------------------------------------------------------------------
;  Declarations
	UDATA_ACS
delay1	res	1	;loop counters for delays
delay2	res	1
delay3	res	1
resHi	res	1	;results registers
resLo	res	1
temp1	res	1	;calculation registers
temp2	res	1

;------------------------------------------------------------------------------
;  Available Resources
	global	Del1ms, Del10ms, Del25ms, Del100ms, DelXms
	global	HexToBCD
	global	resHi, resLo

;------------------------------------------------------------------------------
;  Module Code
misc	code

;------------------------------------------------------------------------------
;  Delays
;  - delay for the specified time interval
;  - these routines do not use the timers, simple loop delays
;  - loops calculated for 16MHz system clock
;  - temp1, temp2, temp3 contents zeroed
;
Del1ms	movlw	0x29	;delay for 1000us = 1ms
	movwf	delay1,A
	movlw	0x06
	movwf	delay2,A
DL1	decfsz	delay1,F,A
	 goto	DL1
	decfsz	delay2,F,A
	 goto	DL1
	return

DelXms	movwf	delay3,A	;delay for specified time in milliseconds (X>0)
DL3	call	Del1ms
	decfsz	delay3,F,A
	 goto	DL3
	return

Del10ms	movlw	0x0A	;delay for 10ms
	goto	DelXms
	
Del25ms	movlw	0x19	;delay for 25ms
	goto	DelXms

Del100ms	movlw	0x64	;delay for 100ms
	goto	DelXms
	

;------------------------------------------------------------------------------
;  HexToBCD
;
;  - convert a single hexadecimal byte to binary coded decimal
;  - enter with hex in W register
;  - exit with result in resHi and resLo
;
HexToBCD	movwf	temp1,A	;convert number to BCD
	clrf	resHi,A	;clear results area
	clrf	resLo,A	
	movlw	0x08	;set loop counter
	movwf	temp2,A
H2BLoop	movf	resLo,W,A	;check 2SD for >=5
	sublw	0x4F
	btfsc	STATUS,C,A
	 goto	H2BD3
	movlw	0x30	;add 3 to upper nibble
	addwf	resLo,F,A
H2BD3	movlw	0x0F	;check LSD for >=5
	andwf	resLo,W,A
	sublw	0x04
	btfsc	STATUS,C,A
	 goto	H2BDL
	movlw	0x03
	addwf	resLo,F,A
H2BDL	rlcf	temp1,F,A	;shift left
	rlcf	resLo,F,A
	rlcf	resHi,F,A
	decfsz	temp2,F,A	;loop 8 times
	 goto	H2BLoop
	movf	resLo,W,A
	return
	
	end
;-- end misc.asm --------------------------------------------------------------
