;------------------------------------------------------------------------------
;  parallel.asm
;
;  - Input and output to the parallel I/O on the SVC connectors for GenPIC
;
;  Andrew Cooper
;  http://www.darkerview.com
;  Creative Commons CC BY-SA 4.0 (Attribution-ShareAlike)
;
;  7Jun2017 A.Cooper
;  - initial version
;
;------------------------------------------------------------------------------

;------------------------------------------------------------------------------
;  Global Definitions
	include	PIC18F66K80.inc ;processor specific definitions
	include	"defs.asm"

;------------------------------------------------------------------------------
;  Available Resources
	global	ParInit
	global	ParWrite0, ParWrite1, ParWrite2, ParWrite3
	global	ParWrite4, ParWrite5
	global	ParRead0, ParRead1, ParRead2, ParRead3
	global	ParRead4, ParRead5

;------------------------------------------------------------------------------
;  Module Code
Parallel	code

;------------------------------------------------------------------------------
;  ParInit
;
;  - set up the parallel bus for writing on the SVC connectors
;  - all select lines set as outputs and high
;
ParInit	clrf	dataDir,A	;set direction lines
	bcf	sel02Dir,sel0,A
	bcf	sel02Dir,sel1,A
	bcf	sel02Dir,sel2,A
	bcf	sel35Dir,sel3,A
	bcf	sel35Dir,sel4,A
	bcf	sel35Dir,sel5,A
	bcf	sel02Port,sel0,A ;set select lines low
	bcf	sel02Port,sel1,A
	bcf	sel02Port,sel2,A
	bcf	sel35Port,sel3,A
	bcf	sel35Port,sel4,A
	bcf	sel35Port,sel5,A
	return

;------------------------------------------------------------------------------
;  ParWriteX
;
;  - write a byte to the parallel bus on the selected SVC port
;  - put the byte on the bus and strobe the select line high
;  - data supplied in W
;
ParWrite0	movwf	dataPort,A	;put the char on the data bus
	bsf	sel02Port,sel0,A ;strobe the sel 0 line
	bcf	sel02Port,sel0,A
	return
ParWrite1	movwf	dataPort,A	;put the char on the data bus
	bsf	sel02Port,sel1,A ;strobe the sel 1 line
	bcf	sel02Port,sel1,A
	return
ParWrite2	movwf	dataPort,A	;put the char on the data bus
	bsf	sel02Port,sel2,A ;strobe the sel 2 line
	bcf	sel02Port,sel2,A
	return
ParWrite3	movwf	dataPort,A	;put the char on the data bus
	bsf	sel35Port,sel3,A ;strobe the sel 3 line
	bcf	sel35Port,sel3,A
	return
ParWrite4	movwf	dataPort,A	;put the char on the data bus
	bsf	sel35Port,sel4,A ;strobe the sel 4 line
	bcf	sel35Port,sel4,A
	return
ParWrite5	movwf	dataPort,A	;put the char on the data bus
	bsf	sel35Port,sel5,A ;strobe the sel 5 line
	bcf	sel35Port,sel5,A
	return

;------------------------------------------------------------------------------
;  ParReadX
;
;  - read a byte from the parallel bus using the selected select line
;  - return the data in W 
;
ParRead0	movlw	0xFF	;set bus as input
	movwf	dataDir,A
	bcf	sel02Port,sel0,A ;strobe the sel 0 line
	nop
	movf	dataPort,W,A	;get the data
	bsf	sel02Port,sel0,A
	clrf	dataDir,A
	return
ParRead1	movlw	0xFF	;set bus as input
	movwf	dataDir,A
	bcf	sel02Port,sel1,A ;strobe the sel 0 line
	nop
	movf	dataPort,W,A	;get the data
	bsf	sel02Port,sel1,A
	clrf	dataDir,A
	return
ParRead2	movlw	0xFF	;set bus as input
	movwf	dataDir,A
	bcf	sel02Port,sel2,A ;strobe the sel 0 line
	nop
	movf	dataPort,W,A	;get the data
	bsf	sel02Port,sel2,A
	clrf	dataDir,A
	return
ParRead3	movlw	0xFF	;set bus as input
	movwf	dataDir,A
	bcf	sel35Port,sel3,A ;strobe the sel 0 line
	nop
	movf	dataPort,W,A	;get the data
	bsf	sel35Port,sel3,A
	clrf	dataDir,A
	return
ParRead4	movlw	0xFF	;set bus as input
	movwf	dataDir,A
	bcf	sel35Port,sel4,A ;strobe the sel 0 line
	nop
	movf	dataPort,W,A	;get the data
	bsf	sel35Port,sel4,A
	clrf	dataDir,A
	return
ParRead5	movlw	0xFF	;set bus as input
	movwf	dataDir,A
	bcf	sel35Port,sel5,A ;strobe the sel 0 line
	nop
	movf	dataPort,W,A	;get the data
	bsf	sel35Port,sel5,A
	clrf	dataDir,A
	return

	end
;- end parallel.asm -----------------------------------------------------------