;------------------------------------------------------------------------------
;  SSP
;
;  - handle the MSSP serial port on the PIC18F66K80
;
;  Andrew Cooper
;  http://www.darkerview.com
;  CC BY-SA 4.0 - Attribution-ShareAlike 4.0 International
;
;  12Jun2017 A. Cooper
;  - initial version
;
;------------------------------------------------------------------------------

;------------------------------------------------------------------------------
;  Global Definitions
	include	PIC18F66K80.inc ;processor specific definitions
	include	"defs.asm"

;------------------------------------------------------------------------------
;  Declarations
	UDATA_ACS
tempRX	res	1	;temp storage

;------------------------------------------------------------------------------
;  Available Resources
	global	SSPInit, SSPSend
	global	SSPSend0, SSPSend1, SSPSend2
	global	SSPSend3, SSPSend4, SSPSend5

;------------------------------------------------------------------------------
;  Module Code
ssp	code

;------------------------------------------------------------------------------
;  SSPInit
;  - setup the SPI serial port
;  - current setup (modify for specs on connected device)
;	Master mode
;	SCLK= Fosc/8
;	Transmit on low to high transition (CKP=0 and CKE=1)
;	SDO and SCK are not open drain
;
SSPInit	movlw	b'11000000'	;config SSP
	movwf	SSPSTAT,A
	movlw	b'00101010'
	movwf	SSPCON1,A
	bcf	sspDir,SDO,A	;ensure pin directions set
	bcf	sspDir,SCK,A
	bsf	sspDir,SDI,A
	;bcf	ODCON,SSPOD,A	;ensure pins not open drain
	bcf	sel02Dir,sel0,A ;set the select lines as outputs
	bcf	sel02Dir,sel1,A
	bcf	sel02Dir,sel2,A
	bcf	sel35Dir,sel3,A
	bcf	sel35Dir,sel4,A
	bcf	sel35Dir,sel5,A
	bsf	sel02Port,sel0,A ;set select lines high
	bsf	sel02Port,sel1,A
	bsf	sel02Port,sel2,A
	bsf	sel35Port,sel3,A
	bsf	sel35Port,sel4,A
	bsf	sel35Port,sel5,A
	movlw	0x00	;write zero to SSP
	movwf	SSPBUF
	return

;------------------------------------------------------------------------------
;  SSPSend
;  - send a char with the SPI serial port
;  - enter with send data in W
;  - exit with recieved data in W	
;
SSPSend	btfss	SSPSTAT,BF,A	;wait for previous transmission
	 goto	SSPSend
	movff	SSPBUF,tempRX	;get recieved char
	movwf	SSPBUF,A
	movf	tempRX,W,A
	return

;------------------------------------------------------------------------------
;  SSPSendX
;  - send a char with the SPI serial port
;  - assert the specified select line low during transfer 
;  - enter with send data in W
;  - exit with recieved data in W	
;
SSPSend0	bcf	sel02Port,sel0,A
	call	SSPSend
	bsf	sel02Port,sel0,A
	return
SSPSend1	bcf	sel02Port,sel1,A
	call	SSPSend
	bsf	sel02Port,sel1,A
	return
SSPSend2	bcf	sel02Port,sel2,A
	call	SSPSend
	bsf	sel02Port,sel2,A
	return
SSPSend3	bcf	sel35Port,sel3,A
	call	SSPSend
	bsf	sel35Port,sel3,A
	return
SSPSend4	bcf	sel35Port,sel4,A
	call	SSPSend
	bsf	sel35Port,sel4,A
	return
SSPSend5	bcf	sel35Port,sel5,A
	call	SSPSend
	bsf	sel35Port,sel5,A
	return

	end
;-- end ssp.asm ---------------------------------------------------------------



