;------------------------------------------------------------------------------
;  TimeBase
;
;  - use timer 2 to generate the main clock
;  - keeps time in hundreths of a second, seconds, minutes, and hours
;  - requires use of the interrupt routine to increment, code in the interrupt
;  routine will check for a timer 2 overflow and set 
;  - If using a 16MHz crytal Fosc/4 (Instruction clock) is 4MHz
;  - Prescaler set for /16 thus count freq is 250000Hz
;  - Comparator set for 250, rollover is 1000Hz
;  - Postscaler set for 1, TMR2IF now at 1000Hz
;
;  Andrew Cooper
;  http://www.darkerview.com
;  CC BY-SA 4.0 - Attribution-ShareAlike 4.0 International
;
;  14Jan2017 A. Cooper
;  - initial version
;
;------------------------------------------------------------------------------

;------------------------------------------------------------------------------
;  Global Definitions
	include	PIC18F66K80.inc ;processor specific definitions
	include	"defs.asm"

;------------------------------------------------------------------------------
;  Declarations
	UDATA_ACS
timeMil	res	1	;timebase registers
timeCent	res	1
timeSec	res	1
timeMin	res	1
timeHour	res	1
timeDay	res	1

;------------------------------------------------------------------------------
;  Available Resources
	global	TimeInit, TimeKeep
	global	timeMil, timeCent, timeSec
	global	timeMin, timeHour, timeDay

;------------------------------------------------------------------------------
;  External Resources
	extern	userPB1Time, userPB2Time, stateFlag

;------------------------------------------------------------------------------
;  Global Definitions
	include	"defs.asm"

;------------------------------------------------------------------------------
;  Module Code
timeBase	code

;------------------------------------------------------------------------------
;  TimeInit
;  - setup timer 2 for use generating clock
;  - timer control register T2CON set as follows...
;	0 not used
;	0 postscaler= /1
;	0
;	0
;	0
;	1 timer 2 on
;	1 prescaler= /16
;	1
;  - enable interrupts on timer 2 overflow
;
TimeInit	movlw	0xFA	;comaprator set to 250
	movwf	PR2,A
	movlw	b'00000111'	;setup timer as above
	movwf	T2CON,A
	bsf	PIE1,TMR2IE,A ;enable timer 2 interrupt
	clrf	timeMil,A	;reset time to zero
	clrf	timeCent,A
	clrf	timeSec,A
	clrf	timeMin,A
	clrf	timeHour,A
	clrf	timeDay,A
	return


;------------------------------------------------------------------------------
;  TimeKeep
;  - update all timer registers
;  - routine must be called every time through the main loop
;  - interval between calls must never be greater than 50ms
;  - timeMil is incremented at 1kHz by the TMR2 module
;
TimeKeep	movf	timeMil,w,A	;check for 0.1 second interval
	sublw	0x09
	btfsc	STATUS,C,A
	 return
	clrf	timeMil,A
	incf	timeCent,f,A	;increment 0.1 second stuff
	incf	userPB1Time,F,A ;increment pushbutton timers
	incf	userPB2Time,F,A
	
	movf	timeCent,w,A ;check for 1 second interval
	sublw	0x63
	btfsc	STATUS,C,A
	 return
	clrf	timeCent,A
	incf	timeSec,f,A	;increment 1 second stuff
	bsf	stateFlag,stateUp,A ;update state display

	movf	timeSec,w,A	;check for 60 seconds
	sublw	0x3B
	btfsc	STATUS,C
	 return
	clrf	timeSec,A
	incf	timeMin,f,A	;increment 1 minute stuff
			;add extra timers here

	movf	timeMin,w,A	;check for 60 minutes
	sublw	0x3B
	btfsc	STATUS,C,A
	 return
	clrf	timeMin,A
	incf	timeHour,f,A ;increment 1 hour stuff
	
	movf	timeHour,w,A ;check for 24 hours
	sublw	0x17
	btfsc	STATUS,C,A
	 return
	clrf	timeHour,A
	incf	timeDay,f,A ;increment days

	movf	timeDay,w,A ;check for 30 days
	sublw	0x1E
	btfsc	STATUS,C,A
	 return
	reset	;reset every 30 days
	
	end
;-- end timebase.asm ----------------------------------------------------------
